/*********************************************************************
 * 
 *    Filename:  getOutput.c
 *    Author:    Ryan Shillington <shillington@writeme.com>
 *    Description: getOutput(filename, output) will search for the program
 *                "filename", and put it's output in the buffer "output".
 *  
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 ********************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>

#define BUF_SIZE 256

void child(char *filename, char *const argv[]);
void parent(char output[BUF_SIZE]);
  
int backup_stdin, backup_stdout;

//* "output" gets filled with the output from executing "filename"
void getOutput(char *filename, char *const argv[], char output[BUF_SIZE])
{
  int filedes[2];
  int whoami;

  //* Backup the original stdin and stdout
  backup_stdin = dup(0);
  backup_stdout = dup(1);

  //* Close the originals
  close(0);
  close(1);

  //* Create a pipe (over top of stdin and stdout) so that we can 
  //* talk to the program 
  if(pipe(filedes) != 0)
  {
    printf("ERROR: COULD NOT OPEN PIPE!!!\n");
    exit(1);
  }
  
  //* Fork, so that the child can be turned into the program that we want
  whoami = fork();

  if(whoami ==0)
  {
    //* This is the child's execution path
    child(filename, argv);
    exit(0);
  } else {
    //* This is the parent's execution path
    parent(output);
    return;
  }
}
  
void parent(char output[BUF_SIZE])
{
  int size;
  //* Get the data comming in
  size = read(0, output, BUF_SIZE);
  output[size] = '\0';
  
  //* Return stdin and stdout to what they were before
  close(0); close(1);
  dup(backup_stdin); dup(backup_stdout);
}

void child(char *filename, char *const argv[])
{
  execvp(filename, argv);
  printf("Command would not execute. Errno = %d, filename = %s\n", errno,
      filename);
}
